Enum brilt::rvsdg::Rvsdg

source ·
pub enum Rvsdg {
    Simple {
        outputs: Vec<Expr>,
    },
    StateFul {
        outputs: Vec<StateExpr>,
        side_effect: Option<StateExpr>,
    },
    Linear(Vec<Rvsdg>),
    BranchIf {
        cond_index: usize,
        branches: [Box<Rvsdg>; 2],
    },
    BranchSwitch {
        cond_index: usize,
        branches: Vec<Rvsdg>,
    },
    Loop {
        body: Box<Rvsdg>,
        cond_index: usize,
        outputs: Vec<usize>,
    },
}

Variants



Simple

Fields

outputs: Vec<Expr>


StateFul

Fields

outputs: Vec<StateExpr>

one of the outputs is not a StateExpr::Arg if side_effect is None

side_effect: Option<StateExpr>

Some() if the stateful operation hasn’t return value

Original RVSDG has state edges but I’ve chosen to introduce a new node type that has single stateful operation and its order is preserved by the linear node to (hopefully) make the representation egglog friendly.



Linear(Vec<Rvsdg>)



BranchIf

Fields

cond_index: usize
branches: [Box<Rvsdg>; 2]

then else

Gamma node cond_var is a bool



BranchSwitch

Fields

cond_index: usize
branches: Vec<Rvsdg>

cond_var is an int



Loop

Fields

body: Box<Rvsdg>
cond_index: usize
outputs: Vec<usize>

Theta node

Implementations

source

impl Rvsdg

source

pub fn new(args: &[String], structure: StructureAnalysis) -> Self

source

pub fn from_egglog(term: &Term, nodes: &[Term]) -> Self

source

pub fn to_bril(&self, args: &[Argument]) -> Vec<Code>